# SINR (Signal to Interference plus Noise Ratio)
# https://opinionitech.com/2021/05/07/rssi-rsrp-rsrq-sinr-cosa-sono-e-come-intepretarli/

import logging
from enum import Enum

from snmp import Engine, SNMPv2c
from snmp.manager import Timeout

from mev_common.models import LteStatus

log = logging.getLogger(__name__)


class LteSignal(str, Enum):
    unknown = "unknown"
    bad = "bad"
    sufficient = "sufficient"
    good = "good"
    excellent = "excellent"


class Snmp:
    def __init__(self, ip="127.0.0.1"):
        self.ip = ip

    def get_lte_signal(self) -> LteStatus:
        try:
            with Engine(SNMPv2c, defaultCommunity=b"public") as engine:
                self.mk = engine.Manager(self.ip)
                result = self.mk.get(
                    "1.3.6.1.4.1.14988.1.1.16.1.1.2.4",  # rssi
                    "1.3.6.1.4.1.14988.1.1.16.1.1.3.4",  # rsrq
                    "1.3.6.1.4.1.14988.1.1.16.1.1.4.4",  # rsrp
                    "1.3.6.1.4.1.14988.1.1.16.1.1.7.4",  # sinr
                    timeout=3
                )
                self.rssi = result[0].value.value
                self.rsrq = result[1].value.value
                self.rsrp = result[2].value.value
                self.sinr = result[3].value.value
        except Exception as e:
            log.warning("snmp error")
            return LteStatus()

        log.debug(
            f"rssi={self.rssi} rsrp={self.rsrp} rsrq={self.rsrq} sinr={self.sinr}"
        )

        lte_status = LteStatus(sinr=self.sinr)

        if self.sinr >= 20:
            lte_status.status = "excellent"
        elif self.sinr >= 13:
            lte_status.status = "good"
        elif self.sinr > 3:
            lte_status.status = "sufficient"
        else:
            lte_status.status = "bad"
        return lte_status


if __name__ == "__main__":
    s = Snmp(ip="10.1.109.185")
    print(s.query())
